(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("ConsentService", ["$q", "Endpoints", ConsentService])
    ;

    function ConsentService($q, endpoints) {
        var service = this;

        service.getWeChatConsentByAccounts = function(accounts, includeContentType) {
            var deferred = $q.defer();
            var accountIds = [];
            var accountIdToNameMap = {};

            accounts.forEach(function(account) {
                accountIds.push(account.Id);
                accountIdToNameMap[account.Id] = account.Name;
            });

            endpoints.fetchMultichannelConsentByAccountIds(accountIds).then(function(rawResp) {
                var consentResp = rawResp.Multichannel_Consent_vod__c;

                // fetch RecordType id for wechat_vod
                endpoints.fetchWeChatRecordType().then(function(recordTypeResp) {
                    var weChatRecordTypeId = recordTypeResp.RecordType[0] ? recordTypeResp.RecordType[0].Id : null;

                    // filter mc consent by record type id to get WeChat consent only
                    var weChatConsentResp = consentResp.filter(function(consentData) {
                        return consentData.RecordTypeId === weChatRecordTypeId;
                    });

                    weChatConsentResp.forEach(function(consent) {
                        if(!!consent.Account_vod__c) {
                            consent.accountName = accountIdToNameMap[consent.Account_vod__c];
                        }
                    });

                    // Look up content type name and attach to the response
                    if(includeContentType) {
                        var contentTypeIds = _.map(_.uniqBy(weChatConsentResp, "Content_Type_vod__c"), "Content_Type_vod__c");
                        endpoints.fetchContentTypeByIds(contentTypeIds).then(function(contentResp) {
                            var contentTypeIdToNameMap = generateIdToNameMap(contentResp.Content_Type_vod__c);
                            weChatConsentResp.map(function(consent) {
                                if(!!consent.Content_Type_vod__c) {
                                    consent.contentName = contentTypeIdToNameMap[consent.Content_Type_vod__c] || '';
                                }
                                return consent;
                            });
                            deferred.resolve(weChatConsentResp);
                        });
                    } else {
                        deferred.resolve(weChatConsentResp);
                    }

                }, deferred.reject);
            }, deferred.reject);
            return deferred.promise;
        };

        function generateIdToNameMap(objectList) {
            var map = {};
            objectList.forEach(function(obj) {
                map[obj.Id] = obj.Name;
            });
            return map;
        }

        return service;
    }
}());